@@ -52,11 +52,14 @@
 #include "panel-applet-marshal.h"
 #include "panel-applet-enums.h"
 
+#define PANEL_APPLET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET, PanelAppletPrivate))
+
 struct _PanelAppletPrivate {
 	PanelAppletShell           *shell;
 	BonoboControl              *control;
 	BonoboPropertyBag          *prop_sack;
 	BonoboItemHandler          *item_handler;
+	GConfClient                *client;
 
 	char                       *iid;
 	GClosure                   *closure;
@@ -104,7 +107,7 @@ enum {
 	PROPERTY_BACKGROUND_IDX,
 	PROPERTY_FLAGS_IDX,
 	PROPERTY_SIZE_HINTS_IDX,
-	PROPERTY_LOCKED_DOWN_IDX,
+	PROPERTY_LOCKED_DOWN_IDX
 };
 
 static void
@@ -184,9 +187,8 @@ panel_applet_add_preferences (PanelApplet  *applet,
 			      const gchar  *schema_dir,
 			      GError      **opt_error)
 {
-	GConfClient  *client;
-	GError      **error = NULL;
-	GError       *our_error = NULL;
+	GError **error = NULL;
+	GError  *our_error = NULL;
 
 	g_return_if_fail (PANEL_IS_APPLET (applet));
 	g_return_if_fail (schema_dir != NULL);
@@ -199,24 +201,17 @@ panel_applet_add_preferences (PanelApplet  *applet,
 	else
 		error = &our_error;
 
-	client = gconf_client_get_default ();
-
 	panel_applet_associate_schemas_in_dir (
-		client, applet->priv->prefs_key, schema_dir, error);
+		applet->priv->client, applet->priv->prefs_key, schema_dir, error);
 
 	if (!opt_error && our_error) {
 		g_warning (G_STRLOC ": failed to add preferences from '%s' : '%s'",
 			   schema_dir, our_error->message);
 		g_error_free (our_error);
 	}
-
-	gconf_client_add_dir (client,
-			      applet->priv->prefs_key,
-			      GCONF_CLIENT_PRELOAD_RECURSIVE,
-			      NULL);
 }
 
-gchar *
+char *
 panel_applet_get_preferences_key (PanelApplet *applet)
 {
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
@@ -227,6 +222,31 @@ panel_applet_get_preferences_key (PanelApplet *applet)
 	return g_strdup (applet->priv->prefs_key);
 }
 
+static void
+panel_applet_set_preferences_key (PanelApplet *applet,
+				  const char  *prefs_key)
+{
+	g_return_if_fail (PANEL_IS_APPLET (applet));
+
+	if (applet->priv->prefs_key) {
+		gconf_client_remove_dir (applet->priv->client,
+					 applet->priv->prefs_key,
+					 NULL);
+
+		g_free (applet->priv->prefs_key);
+		applet->priv->prefs_key = NULL;
+	}
+
+	if (prefs_key) {
+		applet->priv->prefs_key = g_strdup (prefs_key);
+
+		gconf_client_add_dir (applet->priv->client,
+				      applet->priv->prefs_key,
+				      GCONF_CLIENT_PRELOAD_RECURSIVE,
+				      NULL);
+	}
+}
+
 PanelAppletFlags
 panel_applet_get_flags (PanelApplet *applet)
 {
@@ -254,9 +274,12 @@ panel_applet_set_size_hints (PanelApplet      *applet,
 			     int               base_size)
 {
 	CORBA_sequence_CORBA_long *seq;
+	CORBA_Environment          env;
 	CORBA_any                  any;
 	int                        i;
 
+	CORBA_exception_init (&env);
+
 	seq = CORBA_sequence_CORBA_long__alloc ();
 	seq->_length = seq->_maximum = n_elements;
 	seq->_release = CORBA_TRUE;
@@ -272,9 +295,11 @@ panel_applet_set_size_hints (PanelApplet      *applet,
 	Bonobo_PropertyBag_setValue (BONOBO_OBJREF (applet->priv->prop_sack),
 				     PROPERTY_SIZE_HINTS,
 				     &any,
-				     NULL);
+				     &env);
 
 	CORBA_free (seq);
+
+	CORBA_exception_free (&env);
 }
 
 guint
@@ -372,9 +397,16 @@ panel_applet_finalize (GObject *object)
 {
 	PanelApplet *applet = PANEL_APPLET (object);
 
+	panel_applet_set_preferences_key (applet, NULL);
+
+	if (applet->priv->client)
+		g_object_unref (applet->priv->client);
+	applet->priv->client = NULL;
+
 	if (applet->priv->prop_sack)
 		bonobo_object_unref (
 			BONOBO_OBJECT (applet->priv->prop_sack));
+	applet->priv->prop_sack = NULL;
 
 	g_free (applet->priv->size_hints);
 	g_free (applet->priv->prefs_key);
@@ -385,9 +417,6 @@ panel_applet_finalize (GObject *object)
 		g_closure_unref (applet->priv->closure);
 	applet->priv->closure = NULL;
 
-	g_free (applet->priv);
-	applet->priv = NULL;
-
 	parent_class->finalize (object);
 }
 
@@ -1166,7 +1195,7 @@ panel_applet_item_handler_get_object (BonoboItemHandler *handler,
 			continue;
 
 		if (!strcmp (option->key, "prefs_key") && !applet->priv->prefs_key)
-			applet->priv->prefs_key = g_strdup (option->value);
+			panel_applet_set_preferences_key (applet, option->value);
 
 		else if (!strcmp (option->key, "background"))
 			bonobo_pbclient_set_string (BONOBO_OBJREF (applet->priv->prop_sack),
@@ -1274,8 +1303,7 @@ add_tab_bindings (GtkBindingSet   *binding_set,
 }
 
 static void
-panel_applet_class_init (PanelAppletClass *klass,
-			 gpointer          dummy)
+panel_applet_class_init (PanelAppletClass *klass)
 {
 	GObjectClass   *gobject_class = (GObjectClass *) klass;
 	GtkObjectClass *object_class = (GtkObjectClass *) klass;
@@ -1295,6 +1323,8 @@ panel_applet_class_init (PanelAppletClass *klass,
 
 	gobject_class->finalize = panel_applet_finalize;
 
+	g_type_class_add_private (klass, sizeof (PanelAppletPrivate));
+
 	panel_applet_signals [CHANGE_ORIENT] =
                 g_signal_new ("change_orient",
                               G_TYPE_FROM_CLASS (klass),
@@ -1302,10 +1332,10 @@ panel_applet_class_init (PanelAppletClass *klass,
                               G_STRUCT_OFFSET (PanelAppletClass, change_orient),
                               NULL,
 			      NULL,
-                              panel_applet_marshal_VOID__INT,
+                              panel_applet_marshal_VOID__UINT,
                               G_TYPE_NONE,
 			      1,
-			      G_TYPE_INT);
+			      G_TYPE_UINT);
 
 	panel_applet_signals [CHANGE_SIZE] =
                 g_signal_new ("change_size",
@@ -1326,11 +1356,11 @@ panel_applet_class_init (PanelAppletClass *klass,
                               G_STRUCT_OFFSET (PanelAppletClass, change_background),
                               NULL,
 			      NULL,
-                              panel_applet_marshal_VOID__ENUM_POINTER_OBJECT,
+                              panel_applet_marshal_VOID__ENUM_BOXED_OBJECT,
                               G_TYPE_NONE,
 			      3,
 			      PANEL_TYPE_PANEL_APPLET_BACKGROUND_TYPE,
-			      G_TYPE_POINTER,
+			      GDK_TYPE_COLOR,
 			      GDK_TYPE_PIXMAP);
 
 	panel_applet_signals [MOVE_FOCUS_OUT_OF_APPLET] =
@@ -1356,13 +1386,14 @@ static void
 panel_applet_instance_init (PanelApplet      *applet,
 			    PanelAppletClass *klass)
 {
-	applet->priv = g_new0 (PanelAppletPrivate, 1);
+	applet->priv = PANEL_APPLET_GET_PRIVATE (applet);
 
-	applet->priv->bound        = FALSE;
+	applet->priv->client = gconf_client_get_default ();
 
-	applet->priv->flags        = PANEL_APPLET_FLAGS_NONE;
-	applet->priv->orient       = PANEL_APPLET_ORIENT_UP;
-	applet->priv->size         = GNOME_Vertigo_PANEL_MEDIUM;
+	applet->priv->bound  = FALSE;
+	applet->priv->flags  = PANEL_APPLET_FLAGS_NONE;
+	applet->priv->orient = PANEL_APPLET_ORIENT_UP;
+	applet->priv->size   = GNOME_Vertigo_PANEL_MEDIUM;
 
 	applet->priv->moving_focus_out = FALSE;
 
